@@ -31,6 +31,7 @@
 #include <linux/bootmem.h>
 #include <linux/pci.h>
 #include <linux/seq_file.h>
+#include <linux/root_dev.h>
 
 #ifdef CONFIG_BLK_DEV_INITRD
 #include <linux/blk.h>
@@ -608,7 +609,7 @@ setup_arch(char **cmdline_p)
 #endif
 
 	/* Default root filesystem to sda2.  */
-	ROOT_DEV = to_kdev_t(0x0802);
+	ROOT_DEV = Root_SDA2;
 
  	/*
 	 * Check ASN in HWRPB for validity, report if bad.
@@ -1108,7 +1109,7 @@ show_cpuinfo(struct seq_file *f, void *slot)
 #ifdef CONFIG_SMP
 	seq_printf(f, "cpus active\t\t: %d\n"
 		      "cpu active mask\t\t: %016lx\n",
-		       smp_num_cpus, cpu_present_mask);
+		       num_online_cpus(), cpu_present_mask);
 #endif
 
 	return 0;
