@@ -221,8 +221,6 @@ MODULE_PARM(ttymajor, "i");
 MODULE_PARM(calloutmajor, "i");
 MODULE_PARM(verbose, "i");
 
-EXPORT_NO_SYMBOLS;
-
 struct mxser_hwconf {
 	int board_type;
 	int ports;
@@ -652,7 +650,7 @@ int mxser_init(void)
 		n = (sizeof(mxser_pcibrds) / sizeof(mxser_pcibrds[0])) - 1;
 		index = 0;
 		for (b = 0; b < n; b++) {
-			while (pdev = pci_find_device(mxser_pcibrds[b].vendor, mxser_pcibrds[b].device, pdev)) 
+			while ((pdev = pci_find_device(mxser_pcibrds[b].vendor, mxser_pcibrds[b].device, pdev)))
 			{
 				if (pci_enable_device(pdev))
 					continue;
