@@ -1,5 +1,5 @@
 /*======================================================================
-    fmvj18x_cs.c 2.8 2002/03/23
+    fmvj18x_cs.c 2.6 2001/09/17
 
     A fmvj18x (and its compatibles) PCMCIA client driver
 
@@ -29,7 +29,7 @@
 ======================================================================*/
 
 #define DRV_NAME	"fmvj18x_cs"
-#define DRV_VERSION	"2.8"
+#define DRV_VERSION	"2.6"
 
 #include <linux/module.h>
 #include <linux/kernel.h>
@@ -65,10 +65,6 @@
 /*====================================================================*/
 
 /* Module parameters */
-
-MODULE_DESCRIPTION("fmvj18x and compatible PCMCIA ethernet driver");
-MODULE_LICENSE("GPL");
-
 #define INT_MODULE_PARM(n, v) static int n = v; MODULE_PARM(n, "i")
 
 /* Bit map of interrupts to choose from */
@@ -84,7 +80,7 @@ INT_MODULE_PARM(sram_config, 0);
 #ifdef PCMCIA_DEBUG
 INT_MODULE_PARM(pc_debug, PCMCIA_DEBUG);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
-static char *version = DRV_NAME ".c " DRV_VERSION " 2002/03/23";
+static char *version = DRV_NAME ".c " DRV_VERSION " 2001/09/17";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -95,7 +91,6 @@ static char *version = DRV_NAME ".c " DRV_VERSION " 2002/03/23";
  */
 static void fmvj18x_config(dev_link_t *link);
 static int fmvj18x_get_hwinfo(dev_link_t *link, u_char *node_id);
-static int fmvj18x_setup_mfc(dev_link_t *link);
 static void fmvj18x_release(u_long arg);
 static int fmvj18x_event(event_t event, int priority,
 			  event_callback_args_t *args);
@@ -115,7 +110,7 @@ static void fjn_reset(struct net_device *dev);
 static struct net_device_stats *fjn_get_stats(struct net_device *dev);
 static void set_rx_mode(struct net_device *dev);
 static void fjn_tx_timeout(struct net_device *dev);
-static struct ethtool_ops netdev_ethtool_ops;
+static int fjn_ioctl(struct net_device *, struct ifreq *, int);
 
 static dev_info_t dev_info = "fmvj18x_cs";
 static dev_link_t *dev_list;
@@ -127,6 +122,8 @@ typedef enum { MBH10302, MBH10304, TDK, CONTEC, LA501, UNGERMANN,
 	       XXX10304
 } cardtype_t;
 
+#define MANFID_UNGERMANN 0x02c0
+
 /*
     driver specific data structure
 */
@@ -326,7 +323,7 @@ static dev_link_t *fmvj18x_attach(void)
     dev->tx_timeout = fjn_tx_timeout;
     dev->watchdog_timeo = TX_TIMEOUT;
 #endif
-    SET_ETHTOOL_OPS(dev, &netdev_ethtool_ops);
+    dev->do_ioctl = fjn_ioctl;
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -365,7 +362,7 @@ static void fmvj18x_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
-    del_timer_sync(&link->release);
+    del_timer(&link->release);
     if (link->state & DEV_CONFIG) {
 	fmvj18x_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -391,45 +388,6 @@ static void fmvj18x_detach(dev_link_t *link)
 #define CS_CHECK(fn, args...) \
 while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
-static int mfc_try_io_port(dev_link_t *link)
-{
-    int i, ret;
-    static ioaddr_t serial_base[5] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8, 0x0 };
-
-    for (i = 0; i < 5; i++) {
-	link->io.BasePort2 = serial_base[i];
-	link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
-	if (link->io.BasePort2 == 0) {
-	    link->io.NumPorts2 = 0;
-	    printk(KERN_NOTICE "fmvj18x_cs: out of resource for serial\n");
-	}
-	ret = CardServices(RequestIO, link->handle, &link->io);
-	if (ret == CS_SUCCESS) return ret;
-    }
-    return ret;
-}
-
-static int ungermann_try_io_port(dev_link_t *link)
-{
-    int ret;
-    ioaddr_t ioaddr;
-    /*
-	Ungermann-Bass Access/CARD accepts 0x300,0x320,0x340,0x360
-	0x380,0x3c0 only for ioport.
-    */
-    for (ioaddr = 0x300; ioaddr < 0x3e0; ioaddr += 0x20) {
-	link->io.BasePort1 = ioaddr;
-	ret = CardServices(RequestIO, link->handle, &link->io);
-	if (ret == CS_SUCCESS) {
-	    /* calculate ConfigIndex value */
-	    link->conf.ConfigIndex = 
-		((link->io.BasePort1 & 0x0f0) >> 3) | 0x22;
-	    return ret;
-	}
-    }
-    return ret;	/* RequestIO failed */
-}
-
 static void fmvj18x_config(dev_link_t *link)
 {
     client_handle_t handle = link->handle;
@@ -443,7 +401,7 @@ static void fmvj18x_config(dev_link_t *link)
     cardtype_t cardtype;
     char *card_name = "unknown";
     u_char *node_id;
-
+    
     DEBUG(0, "fmvj18x_config(0x%p)\n", link);
 
     /*
@@ -486,11 +444,6 @@ static void fmvj18x_config(dev_link_t *link)
 		CardServices(GetStatus, handle, &status);
 		if (status.CardState & CS_EVENT_3VCARD)
 		    link->conf.Vcc = 33; /* inserted in 3.3V slot */
-	    } else if (le16_to_cpu(buf[1]) == PRODID_TDK_GN3410) {
-		/* MultiFunction Card */
-		link->conf.ConfigBase = 0x800;
-		link->conf.ConfigIndex = 0x47;
-		link->io.NumPorts2 = 8;
 	    }
 	    break;
 	case MANFID_CONTEC:
@@ -528,24 +481,31 @@ static void fmvj18x_config(dev_link_t *link)
 	    break;
 	case MANFID_UNGERMANN:
 	    cardtype = UNGERMANN;
-	    break;
+	    /*
+	       Ungermann-Bass Access/CARD accepts 0x300,0x320,0x340,0x360
+	       0x380,0x3c0 only for ioport.
+	    */
+	    for (link->io.BasePort1 = 0x300; link->io.BasePort1 < 0x3e0;
+		link->io.BasePort1 += 0x20) {
+		ret = CardServices(RequestIO, link->handle, &link->io);
+		if (ret == CS_SUCCESS) {
+		/* calculate ConfigIndex value */
+		link->conf.ConfigIndex = 
+		    ((link->io.BasePort1 & 0x0f0) >> 3) | 0x22;
+		goto req_irq;
+		}
+	    }
+	    /* if ioport allocation is failed, goto failed */
+	    printk(KERN_NOTICE "fmvj18x_cs: register_netdev() failed\n");
+	    goto failed;
 	default:
 	    cardtype = MBH10302;
 	    link->conf.ConfigIndex = 1;
 	}
     }
 
-    if (link->io.NumPorts2 != 0) {
-    	link->irq.Attributes =
-		IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
-	ret = mfc_try_io_port(link);
-	if (ret != CS_SUCCESS) goto cs_failed;
-    } else if (cardtype == UNGERMANN) {
-	ret = ungermann_try_io_port(link);
-	if (ret != CS_SUCCESS) goto cs_failed;
-    } else { 
-	CS_CHECK(RequestIO, link->handle, &link->io);
-    }
+    CS_CHECK(RequestIO, link->handle, &link->io);
+req_irq:
     CS_CHECK(RequestIRQ, link->handle, &link->irq);
     CS_CHECK(RequestConfiguration, link->handle, &link->conf);
     dev->irq = link->irq.AssignedIRQ;
@@ -555,22 +515,19 @@ static void fmvj18x_config(dev_link_t *link)
 	goto failed;
     }
 
-    if (link->io.BasePort2 != 0)
-	fmvj18x_setup_mfc(link);
-
     ioaddr = dev->base_addr;
 
     /* Reset controller */
-    if (sram_config == 0) 
+    if( sram_config == 0 ) 
 	outb(CONFIG0_RST, ioaddr + CONFIG_0);
     else
 	outb(CONFIG0_RST_1, ioaddr + CONFIG_0);
 
     /* Power On chip and select bank 0 */
-    if (cardtype == MBH10302)
-	outb(BANK_0, ioaddr + CONFIG_1);
-    else
+    if(cardtype == UNGERMANN)
 	outb(BANK_0U, ioaddr + CONFIG_1);
+    else
+	outb(BANK_0, ioaddr + CONFIG_1);
     
     /* Set hardware address */
     switch (cardtype) {
@@ -614,7 +571,8 @@ static void fmvj18x_config(dev_link_t *link)
     case XXX10304:
 	/* Read MACID from Buggy CIS */
 	if (fmvj18x_get_hwinfo(link, tuple.TupleData) == -1) {
-	    printk(KERN_NOTICE "fmvj18x_cs: unable to read hardware net address.\n");
+	    printk(KERN_NOTICE "fmvj18x_cs: unable to read hardware net 
+		address.");
 	    unregister_netdev(dev);
 	    goto failed;
 	}
@@ -634,6 +592,7 @@ static void fmvj18x_config(dev_link_t *link)
 
     strcpy(lp->node.dev_name, dev->name);
     link->dev = &lp->node;
+    link->state &= ~DEV_CONFIG_PENDING;
 
     lp->cardtype = cardtype;
     /* print current configuration */
@@ -643,7 +602,6 @@ static void fmvj18x_config(dev_link_t *link)
     for (i = 0; i < 6; i++)
 	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
 
-    link->state &= ~DEV_CONFIG_PENDING;
     return;
     
 cs_failed:
@@ -651,7 +609,6 @@ cs_failed:
     cs_error(link->handle, last_fn, last_ret);
 failed:
     fmvj18x_release((u_long)link);
-    link->state &= ~DEV_CONFIG_PENDING;
 
 } /* fmvj18x_config */
 /*====================================================================*/
@@ -710,51 +667,6 @@ static int fmvj18x_get_hwinfo(dev_link_t *link, u_char *node_id)
 } /* fmvj18x_get_hwinfo */
 /*====================================================================*/
 
-static int fmvj18x_setup_mfc(dev_link_t *link)
-{
-    win_req_t req;
-    memreq_t mem;
-    u_char *base;
-    int i, j;
-    local_info_t *lp = link->priv;
-    struct net_device *dev = &lp->dev;
-    ioaddr_t ioaddr;
-
-    /* Allocate a small memory window */
-    req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_AM|WIN_ENABLE;
-    req.Base = 0; req.Size = 0;
-    req.AccessSpeed = 0;
-    link->win = (window_handle_t)link->handle;
-    i = CardServices(RequestWindow, &link->win, &req);
-    if (i != CS_SUCCESS) {
-	cs_error(link->handle, RequestWindow, i);
-	return -1;
-    }
-
-    base = ioremap(req.Base, req.Size);
-    mem.Page = 0;
-    mem.CardOffset = 0;
-    CardServices(MapMemPage, link->win, &mem);
-
-    ioaddr = dev->base_addr;
-    writeb(0x47, base+0x800);	/* Config Option Register of LAN */
-    writeb(0x0, base+0x802);	/* Config and Status Register */
-
-    writeb(ioaddr & 0xff, base+0x80a);		/* I/O Base(Low) of LAN */
-    writeb((ioaddr >> 8) & 0xff, base+0x80c);	/* I/O Base(High) of LAN */
-   
-    writeb(0x45, base+0x820);	/* Config Option Register of Modem */
-    writeb(0x8, base+0x822);	/* Config and Status Register */
-
-    iounmap(base);
-    j = CardServices(ReleaseWindow, link->win);
-    if (j != CS_SUCCESS)
-	cs_error(link->handle, ReleaseWindow, j);
-    return 0;
-
-}
-/*====================================================================*/
-
 static void fmvj18x_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
@@ -841,7 +753,7 @@ static int __init init_fmvj18x_cs(void)
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "fmvj18x: Card Services release "
 	       "does not match!\n");
-	return -EINVAL;
+	return -1;
     }
     register_pccard_driver(&dev_info, &fmvj18x_attach, &fmvj18x_detach);
     return 0;
@@ -954,19 +866,11 @@ static int fjn_start_xmit(struct sk_buff *skb, struct net_device *dev)
 {
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
     ioaddr_t ioaddr = dev->base_addr;
-    short length = skb->len;
-    
-    if(length < ETH_ZLEN)
-    {
-    	skb = skb_padto(skb, ETH_ZLEN);
-    	if(skb == NULL)
-    		return 0;
-    	length = ETH_ZLEN;
-    }
 
     netif_stop_queue(dev);
 
     {
+	short length = ETH_ZLEN < skb->len ? skb->len : ETH_ZLEN;
 	unsigned char *buf = skb->data;
 
 	if (length > ETH_FRAME_LEN) {
@@ -1039,10 +943,10 @@ static void fjn_reset(struct net_device *dev)
 	outb(CONFIG0_RST_1, ioaddr + CONFIG_0);
 
     /* Power On chip and select bank 0 */
-    if (lp->cardtype == MBH10302)
-	outb(BANK_0, ioaddr + CONFIG_1);
-    else
+    if( lp->cardtype == UNGERMANN)
 	outb(BANK_0U, ioaddr + CONFIG_1);
+    else
+	outb(BANK_0, ioaddr + CONFIG_1);
 
     /* Set Tx modes */
     outb(D_TX_MODE, ioaddr + TX_MODE);
@@ -1054,20 +958,20 @@ static void fjn_reset(struct net_device *dev)
         outb(dev->dev_addr[i], ioaddr + NODE_ID + i);
 
     /* Switch to bank 1 */
-    if (lp->cardtype == MBH10302)
-	outb(BANK_1, ioaddr + CONFIG_1);
-    else
+    if ( lp->cardtype == UNGERMANN )
 	outb(BANK_1U, ioaddr + CONFIG_1);
+    else
+	outb(BANK_1, ioaddr + CONFIG_1);
 
     /* set the multicast table to accept none. */
     for (i = 0; i < 6; i++) 
         outb(0x00, ioaddr + MAR_ADR + i);
 
     /* Switch to bank 2 (runtime mode) */
-    if (lp->cardtype == MBH10302)
-	outb(BANK_2, ioaddr + CONFIG_1);
-    else
+    if ( lp->cardtype == UNGERMANN )
 	outb(BANK_2U, ioaddr + CONFIG_1);
+    else
+	outb(BANK_2, ioaddr + CONFIG_1);
 
     /* set 16col ctrl bits */
     if( lp->cardtype == TDK || lp->cardtype == CONTEC) 
@@ -1096,16 +1000,16 @@ static void fjn_reset(struct net_device *dev)
     outb(0xff, ioaddr + TX_STATUS);
     outb(0xff, ioaddr + RX_STATUS);
 
-    if (lp->cardtype == MBH10302)
-	outb(INTR_OFF, ioaddr + LAN_CTRL);
+    if( lp->cardtype != TDK ) 
+    		outb(INTR_OFF, ioaddr + LAN_CTRL);
 
     /* Turn on Rx interrupts */
     outb(D_TX_INTR, ioaddr + TX_INTR);
     outb(D_RX_INTR, ioaddr + RX_INTR);
 
     /* Turn on interrupts from LAN card controller */
-    if (lp->cardtype == MBH10302)
-	outb(INTR_ON, ioaddr + LAN_CTRL);
+    if( lp->cardtype != TDK ) 
+		outb(INTR_ON, ioaddr + LAN_CTRL);
 } /* fjn_reset */
 
 /*====================================================================*/
@@ -1160,7 +1064,7 @@ static void fjn_rx(struct net_device *dev)
 
 	    skb_reserve(skb, 2);
 	    insw(ioaddr + DATAPORT, skb_put(skb, pkt_len),
-		 (pkt_len + 1) >> 1);
+		    (pkt_len + 1) >> 1);
 	    skb->protocol = eth_type_trans(skb, dev);
 
 #ifdef PCMCIA_DEBUG
@@ -1187,7 +1091,7 @@ static void fjn_rx(struct net_device *dev)
 	   has done a netif_wake_queue() for us and will work on them
 	   when we get to the bottom-half routine. */
 /*
-    if (lp->cardtype != TDK) {
+    if( lp->cardtype != TDK ) {
 	int i;
 	for (i = 0; i < 20; i++) {
 	    if ((inb(ioaddr + RX_MODE) & F_BUF_EMP) == F_BUF_EMP)
@@ -1207,33 +1111,64 @@ static void fjn_rx(struct net_device *dev)
 
 /*====================================================================*/
 
-static void netdev_get_drvinfo(struct net_device *dev,
-			       struct ethtool_drvinfo *info)
+static int netdev_ethtool_ioctl (struct net_device *dev, void *useraddr)
 {
-	strcpy(info->driver, DRV_NAME);
-	strcpy(info->version, DRV_VERSION);
-	sprintf(info->bus_info, "PCMCIA 0x%lx", dev->base_addr);
-}
+	u32 ethcmd;
+
+	/* dev_ioctl() in ../../net/core/dev.c has already checked
+	   capable(CAP_NET_ADMIN), so don't bother with that here.  */
+
+	if (get_user(ethcmd, (u32 *)useraddr))
+		return -EFAULT;
+
+	switch (ethcmd) {
+
+	case ETHTOOL_GDRVINFO: {
+		struct ethtool_drvinfo info = { ETHTOOL_GDRVINFO };
+		strcpy (info.driver, DRV_NAME);
+		strcpy (info.version, DRV_VERSION);
+		sprintf(info.bus_info, "PCMCIA 0x%lx", dev->base_addr);
+		if (copy_to_user (useraddr, &info, sizeof (info)))
+			return -EFAULT;
+		return 0;
+	}
 
 #ifdef PCMCIA_DEBUG
-static u32 netdev_get_msglevel(struct net_device *dev)
-{
-	return pc_debug;
+	/* get message-level */
+	case ETHTOOL_GMSGLVL: {
+		struct ethtool_value edata = {ETHTOOL_GMSGLVL};
+		edata.data = pc_debug;
+		if (copy_to_user(useraddr, &edata, sizeof(edata)))
+			return -EFAULT;
+		return 0;
+	}
+	/* set message-level */
+	case ETHTOOL_SMSGLVL: {
+		struct ethtool_value edata;
+		if (copy_from_user(&edata, useraddr, sizeof(edata)))
+			return -EFAULT;
+		pc_debug = edata.data;
+		return 0;
+	}
+#endif
+
+	default:
+		break;
+	}
+
+	return -EOPNOTSUPP;
 }
 
-static void netdev_set_msglevel(struct net_device *dev, u32 level)
+static int fjn_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
-	pc_debug = level;
-}
-#endif /* PCMCIA_DEBUG */
+	switch (cmd) {
+	case SIOCETHTOOL:
+		return netdev_ethtool_ioctl(dev, (void *) rq->ifr_data);
 
-static struct ethtool_ops netdev_ethtool_ops = {
-	.get_drvinfo		= netdev_get_drvinfo,
-#ifdef PCMCIA_DEBUG
-	.get_msglevel		= netdev_get_msglevel,
-	.set_msglevel		= netdev_set_msglevel,
-#endif /* PCMCIA_DEBUG */
-};
+	default:
+		return -EOPNOTSUPP;
+	}
+}
 
 static int fjn_config(struct net_device *dev, struct ifmap *map){
     return 0;
@@ -1289,7 +1224,7 @@ static int fjn_close(struct net_device *dev)
     outb(CHIP_OFF ,ioaddr + CONFIG_1);
 
     /* Set the ethernet adaptor disable IRQ */
-    if (lp->cardtype == MBH10302)
+    if( lp->cardtype != TDK ) 
 	outb(INTR_OFF, ioaddr + LAN_CTRL);
 
     link->open--;
@@ -1318,8 +1253,8 @@ static void set_rx_mode(struct net_device *dev)
 {
     ioaddr_t ioaddr = dev->base_addr;
     struct local_info_t *lp = (struct local_info_t *)dev->priv;
-    u_char mc_filter[8];		 /* Multicast hash filter */
-    u_long flags;
+    unsigned char mc_filter[8];		 /* Multicast hash filter */
+    unsigned long flags;
     int i;
     
     if (dev->flags & IFF_PROMISC) {
@@ -1359,3 +1294,4 @@ static void set_rx_mode(struct net_device *dev)
     }
     restore_flags(flags);
 }
+MODULE_LICENSE("GPL");
