@@ -113,6 +113,15 @@ static int sonic_send_packet(struct sk_buff *skb, struct net_device *dev)
 	if (sonic_debug > 2)
 		printk("sonic_send_packet: skb=%p, dev=%p\n", skb, dev);
 
+	/* 
+	 * Block a timer-based transmit from overlapping.  This could better be
+	 * done with atomic_swap(1, dev->tbusy), but set_bit() works as well.
+	 */
+	if (test_and_set_bit(0, (void *) &dev->tbusy) != 0) {
+		printk("%s: Transmitter access conflict.\n", dev->name);
+		return 1;
+	}
+
 	/*
 	 * Map the packet data into the logical DMA address space
 	 */
@@ -223,7 +232,7 @@ static void sonic_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 
 			/* We must free the original skb */
 			if (lp->tx_skb[entry]) {
-				dev_kfree_skb_irq(lp->tx_skb[entry]);
+				dev_kfree_skb(lp->tx_skb[entry]);
 				lp->tx_skb[entry] = 0;
 			}
 			/* and the VDMA address */
