@@ -137,7 +137,7 @@ static int sa1100_pcmcia_set_mecr(int sock)
 
 	local_irq_restore(flags);
 
-	DEBUG(4, "%s(): FAST%u %lx  BSM%u %lx  BSA%u %lx  BSIO%u %lx\n",
+	DEBUG(4, "%s(): FAST%u %X  BSM%u %X  BSA%u %X  BSIO%u %X\n",
 	      __FUNCTION__, sock, MECR_FAST_GET(mecr, sock), sock,
 	      MECR_BSM_GET(mecr, sock), sock, MECR_BSA_GET(mecr, sock),
 	      sock, MECR_BSIO_GET(mecr, sock));
@@ -686,7 +686,7 @@ sa1100_pcmcia_get_mem_map(unsigned int sock, struct pccard_mem_map *map)
   DEBUG(2, "%s() for sock %u\n", __FUNCTION__, sock);
 
   if (map->map < MAX_WIN) {
-    *map = skt->mem_map[map->map];
+    *map = skt->pc_mem_map[map->map];
     ret = 0;
   }
 
@@ -754,7 +754,7 @@ sa1100_pcmcia_set_mem_map(unsigned int sock, struct pccard_mem_map *map)
   map->sys_stop += start;
   map->sys_start = start;
 
-  skt->mem_map[map->map] = *map;
+  skt->pc_mem_map[map->map] = *map;
 
   return 0;
 }  /* sa1100_pcmcia_set_mem_map() */
@@ -1052,6 +1052,7 @@ int sa1100_register_pcmcia(struct pcmcia_low_level *ops)
 	pcmcia_low_level = NULL;
 	return ret;
 }
+EXPORT_SYMBOL(sa1100_register_pcmcia);
 
 /* sa1100_unregister_pcmcia()
  * ^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -1091,6 +1092,7 @@ void sa1100_unregister_pcmcia(struct pcmcia_low_level *ops)
 
 	pcmcia_low_level = NULL;
 }
+EXPORT_SYMBOL(sa1100_unregister_pcmcia);
 
 /* sa1100_pcmcia_init()
  * ^^^^^^^^^^^^^^^^^^^^
