@@ -90,6 +90,8 @@
  * memory bank.  For those systems, simply undefine CONFIG_DISCONTIGMEM.
  */
 
+#ifdef CONFIG_ARCH_EDB7211
+
 #ifdef CONFIG_DISCONTIGMEM
 /*
  * Because of the wide memory address space between physical RAM banks on the 
@@ -153,5 +155,11 @@
 
 #endif /* CONFIG_DISCONTIGMEM */
 
+#endif	/* CONFIG_ARCH_EDB7211 */
+
+#ifndef PFN_TO_NID
+#define PFN_TO_NID(pfn)		(0)
+#endif
+
 #endif
 
