@@ -10,6 +10,8 @@
 #ifndef _ASMARM_CACHEFLUSH_H
 #define _ASMARM_CACHEFLUSH_H
 
+#include <linux/sched.h>
+#include <linux/mm.h>
 #include <asm/proc/cache.h>
 
 #endif
