@@ -61,7 +61,7 @@ hashbin_t *ircomm = NULL;
 
 int __init ircomm_init(void)
 {
-	ircomm = hashbin_new(HB_LOCAL); 
+	ircomm = hashbin_new(HB_LOCK); 
 	if (ircomm == NULL) {
 		ERROR(__FUNCTION__ "(), can't allocate hashbin!\n");
 		return -ENOMEM;
@@ -76,8 +76,7 @@ int __init ircomm_init(void)
 	return 0;
 }
 
-#ifdef MODULE
-void ircomm_cleanup(void)
+void __exit ircomm_cleanup(void)
 {
 	IRDA_DEBUG(2, __FUNCTION__ "()\n");
 
@@ -87,7 +86,6 @@ void ircomm_cleanup(void)
 	remove_proc_entry("ircomm", proc_irda);
 #endif /* CONFIG_PROC_FS */
 }
-#endif /* MODULE */
 
 /*
  * Function ircomm_open (client_notify)
@@ -134,6 +132,8 @@ struct ircomm_cb *ircomm_open(notify_t *notify, __u8 service_type, int line)
 	return self;
 }
 
+EXPORT_SYMBOL(ircomm_open);
+
 /*
  * Function ircomm_close_instance (self)
  *
@@ -187,6 +187,8 @@ int ircomm_close(struct ircomm_cb *self)
         return __ircomm_close(self);
 }
 
+EXPORT_SYMBOL(ircomm_close);
+
 /*
  * Function ircomm_connect_request (self, service_type)
  *
@@ -217,6 +219,8 @@ int ircomm_connect_request(struct ircomm_cb *self, __u8 dlsap_sel,
 	return ret;
 }
 
+EXPORT_SYMBOL(ircomm_connect_request);
+
 /*
  * Function ircomm_connect_indication (self, qos, skb)
  *
@@ -269,6 +273,8 @@ int ircomm_connect_response(struct ircomm_cb *self, struct sk_buff *userdata)
 	return ret;
 }	
 
+EXPORT_SYMBOL(ircomm_connect_response);
+
 /*
  * Function connect_confirm (self, skb)
  *
@@ -312,6 +318,8 @@ int ircomm_data_request(struct ircomm_cb *self, struct sk_buff *skb)
 	return ret;
 }
 
+EXPORT_SYMBOL(ircomm_data_request);
+
 /*
  * Function ircomm_data_indication (self, skb)
  *
@@ -387,6 +395,8 @@ int ircomm_control_request(struct ircomm_cb *self, struct sk_buff *skb)
 	return ret;
 }
 
+EXPORT_SYMBOL(ircomm_control_request);
+
 /*
  * Function ircomm_control_indication (self, skb)
  *
@@ -438,6 +448,8 @@ int ircomm_disconnect_request(struct ircomm_cb *self, struct sk_buff *userdata)
 	return ret;
 }
 
+EXPORT_SYMBOL(ircomm_disconnect_request);
+
 /*
  * Function disconnect_indication (self, skb)
  *
@@ -479,6 +491,8 @@ void ircomm_flow_request(struct ircomm_cb *self, LOCAL_FLOW flow)
 	irttp_flow_request(self->tsap, flow);
 }
 
+EXPORT_SYMBOL(ircomm_flow_request);
+
 #ifdef CONFIG_PROC_FS
 /*
  * Function ircomm_proc_read (buf, start, offset, len, unused)
@@ -491,14 +505,13 @@ int ircomm_proc_read(char *buf, char **start, off_t offset, int len)
 	struct ircomm_cb *self;
 	unsigned long flags;
 	
-	save_flags(flags);
-	cli();
-
 	len = 0;
 
+	spin_lock_irqsave(&ircomm->hb_spinlock, flags);
+
 	self = (struct ircomm_cb *) hashbin_get_first(ircomm);
 	while (self != NULL) {
-		ASSERT(self->magic == IRCOMM_MAGIC, return len;);
+		ASSERT(self->magic == IRCOMM_MAGIC, break;);
 
 		if(self->line < 0x10)
 			len += sprintf(buf+len, "ircomm%d", self->line);
@@ -521,25 +534,15 @@ int ircomm_proc_read(char *buf, char **start, off_t offset, int len)
 
 		self = (struct ircomm_cb *) hashbin_get_next(ircomm);
  	} 
-	restore_flags(flags);
+	spin_unlock_irqrestore(&ircomm->hb_spinlock, flags);
 
 	return len;
 }
 #endif /* CONFIG_PROC_FS */
 
-#ifdef MODULE
 MODULE_AUTHOR("Dag Brattli <dag@brattli.net>");
 MODULE_DESCRIPTION("IrCOMM protocol");
 MODULE_LICENSE("GPL");
 
-int init_module(void) 
-{
-	return ircomm_init();
-}
-	
-void cleanup_module(void)
-{
-	ircomm_cleanup();
-}
-#endif /* MODULE */
-
+module_init(ircomm_init);
+module_exit(ircomm_cleanup);
