@@ -1,5 +1,5 @@
 /*********************************************************************
- *                
+ *
  * Filename:      irda_device.c
  * Version:       0.9
  * Description:   Utility functions used by the device drivers
@@ -8,25 +8,25 @@
  * Created at:    Sat Oct  9 09:22:27 1999
  * Modified at:   Sun Jan 23 17:41:24 2000
  * Modified by:   Dag Brattli <dagb@cs.uit.no>
- * 
+ *
  *     Copyright (c) 1999-2000 Dag Brattli, All Rights Reserved.
  *     Copyright (c) 2000-2001 Jean Tourrilhes <jt@hpl.hp.com>
- *     
- *     This program is free software; you can redistribute it and/or 
- *     modify it under the terms of the GNU General Public License as 
- *     published by the Free Software Foundation; either version 2 of 
+ *
+ *     This program is free software; you can redistribute it and/or
+ *     modify it under the terms of the GNU General Public License as
+ *     published by the Free Software Foundation; either version 2 of
  *     the License, or (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *     GNU General Public License for more details.
- * 
- *     You should have received a copy of the GNU General Public License 
- *     along with this program; if not, write to the Free Software 
- *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ *
+ *     You should have received a copy of the GNU General Public License
+ *     along with this program; if not, write to the Free Software
+ *     Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  *     MA 02111-1307 USA
- *     
+ *
  ********************************************************************/
 
 #include <linux/config.h>
@@ -44,7 +44,6 @@
 #include <linux/spinlock.h>
 
 #include <asm/ioctls.h>
-#include <asm/segment.h>
 #include <asm/uaccess.h>
 #include <asm/dma.h>
 #include <asm/io.h>
@@ -56,20 +55,6 @@
 #include <net/irda/timer.h>
 #include <net/irda/wrapper.h>
 
-extern int irtty_init(void);
-extern int nsc_ircc_init(void);
-extern int ircc_init(void);
-extern int toshoboe_init(void);
-extern int litelink_init(void);
-extern int w83977af_init(void);
-extern int esi_init(void);
-extern int tekram_init(void);
-extern int actisys_init(void);
-extern int girbil_init(void);
-extern int sa1100_irda_init(void);
-extern int ep7211_ir_init(void);
-extern int mcp2120_init(void);
-
 static void __irda_task_delete(struct irda_task *task);
 
 static hashbin_t *dongles = NULL;
@@ -85,7 +70,7 @@ const char *infrared_mode[] = {
 #ifdef CONFIG_IRDA_DEBUG
 static const char *task_state[] = {
 	"IRDA_TASK_INIT",
-	"IRDA_TASK_DONE", 
+	"IRDA_TASK_DONE",
 	"IRDA_TASK_WAIT",
 	"IRDA_TASK_WAIT1",
 	"IRDA_TASK_WAIT2",
@@ -99,80 +84,32 @@ static const char *task_state[] = {
 static void irda_task_timer_expired(void *data);
 
 #ifdef CONFIG_PROC_FS
-int irda_device_proc_read(char *buf, char **start, off_t offset, int len, 
+int irda_device_proc_read(char *buf, char **start, off_t offset, int len,
 			  int unused);
 
 #endif /* CONFIG_PROC_FS */
 
 int __init irda_device_init( void)
 {
-	dongles = hashbin_new(HB_GLOBAL);
+	dongles = hashbin_new(HB_LOCK);
 	if (dongles == NULL) {
-		printk(KERN_WARNING 
-		       "IrDA: Can't allocate dongles hashbin!\n");
+		printk(KERN_WARNING "IrDA: Can't allocate dongles hashbin!\n");
 		return -ENOMEM;
 	}
 
-	tasks = hashbin_new(HB_GLOBAL);
+	tasks = hashbin_new(HB_LOCK);
 	if (tasks == NULL) {
-		printk(KERN_WARNING 
-		       "IrDA: Can't allocate tasks hashbin!\n");
+		printk(KERN_WARNING "IrDA: Can't allocate tasks hashbin!\n");
 		return -ENOMEM;
 	}
 
-	/* 
-	 * Call the init function of the device drivers that has not been
-	 * compiled as a module 
-	 * Note : non-modular IrDA is not supported in 2.4.X, so don't
-	 * waste too much time fixing this code. If you require it, please
-	 * upgrade to the IrDA stack in 2.5.X. Jean II
-	 */
-#ifdef CONFIG_IRTTY_SIR
-	irtty_init();
-#endif
-#ifdef CONFIG_WINBOND_FIR
-	w83977af_init();
-#endif
-#ifdef CONFIG_SA1100_FIR
-	sa1100_irda_init();
-#endif
-#ifdef CONFIG_NSC_FIR
-	nsc_ircc_init();
-#endif
-#ifdef CONFIG_TOSHIBA_OLD
-	toshoboe_init();
-#endif
-#ifdef CONFIG_SMC_IRCC_FIR
-	ircc_init();
-#endif
-#ifdef CONFIG_ESI_DONGLE
-	esi_init();
-#endif
-#ifdef CONFIG_TEKRAM_DONGLE
-	tekram_init();
-#endif
-#ifdef CONFIG_ACTISYS_DONGLE
-	actisys_init();
-#endif
-#ifdef CONFIG_GIRBIL_DONGLE
-	girbil_init();
-#endif
-#ifdef CONFIG_LITELINK_DONGLE
-	litelink_init();
-#endif
-#ifdef CONFIG_OLD_BELKIN
- 	old_belkin_init();
-#endif
-#ifdef CONFIG_EP7211_IR
- 	ep7211_ir_init();
-#endif
-#ifdef CONFIG_MCP2120_DONGLE
-	mcp2120_init();
-#endif
+	/* We no longer initialise the driver ourselves here, we let
+	 * the system do it for us... - Jean II */
+
 	return 0;
 }
 
-void irda_device_cleanup(void)
+void __exit irda_device_cleanup(void)
 {
 	IRDA_DEBUG(4, __FUNCTION__ "()\n");
 
@@ -186,7 +123,7 @@ void irda_device_cleanup(void)
  *    Called when we have detected that another station is transmiting
  *    in contention mode.
  */
-void irda_device_set_media_busy(struct net_device *dev, int status) 
+void irda_device_set_media_busy(struct net_device *dev, int status)
 {
 	struct irlap_cb *self;
 
@@ -211,15 +148,15 @@ void irda_device_set_media_busy(struct net_device *dev, int status)
 }
 
 int irda_device_set_dtr_rts(struct net_device *dev, int dtr, int rts)
-{	
+{
 	struct if_irda_req req;
 	int ret;
 
 	IRDA_DEBUG(2, __FUNCTION__ "()\n");
 
 	if (!dev->do_ioctl) {
-		ERROR(__FUNCTION__ "(), do_ioctl not impl. by "
-		      "device driver\n");
+		ERROR("%s: do_ioctl not impl. by device driver\n",
+				__FUNCTION__);
 		return -1;
 	}
 
@@ -232,15 +169,15 @@ int irda_device_set_dtr_rts(struct net_device *dev, int dtr, int rts)
 }
 
 int irda_device_change_speed(struct net_device *dev, __u32 speed)
-{	
+{
 	struct if_irda_req req;
 	int ret;
 
 	IRDA_DEBUG(2, __FUNCTION__ "()\n");
 
 	if (!dev->do_ioctl) {
-		ERROR(__FUNCTION__ "(), do_ioctl not impl. by "
-		      "device driver\n");
+		ERROR("%s: do_ioctl not impl. by device driver\n",
+				__FUNCTION__);
 		return -1;
 	}
 
@@ -265,8 +202,8 @@ int irda_device_is_receiving(struct net_device *dev)
 	IRDA_DEBUG(2, __FUNCTION__ "()\n");
 
 	if (!dev->do_ioctl) {
-		ERROR(__FUNCTION__ "(), do_ioctl not impl. by "
-		      "device driver\n");
+		ERROR("%s: do_ioctl not impl. by device driver\n",
+				__FUNCTION__);
 		return -1;
 	}
 
@@ -287,7 +224,7 @@ void irda_task_next_state(struct irda_task *task, IRDA_TASK_STATE state)
 static void __irda_task_delete(struct irda_task *task)
 {
 	del_timer(&task->timer);
-	
+
 	kfree(task);
 }
 
@@ -322,14 +259,14 @@ int irda_task_kick(struct irda_task *task)
 	do {
 		timeout = task->function(task);
 		if (count++ > 100) {
-			ERROR(__FUNCTION__ "(), error in task handler!\n");
+			ERROR("%s: error in task handler!\n", __FUNCTION__);
 			irda_task_delete(task);
 			return TRUE;
-		}			
+		}
 	} while ((timeout == 0) && (task->state != IRDA_TASK_DONE));
 
 	if (timeout < 0) {
-		ERROR(__FUNCTION__ "(), Error executing task!\n");
+		ERROR("%s: Error executing task!\n", __FUNCTION__);
 		irda_task_delete(task);
 		return TRUE;
 	}
@@ -354,14 +291,14 @@ int irda_task_kick(struct irda_task *task)
 				/* Kick parent task */
 				irda_task_kick(task->parent);
 			}
-		}		
+		}
 		irda_task_delete(task);
 	} else if (timeout > 0) {
-		irda_start_timer(&task->timer, timeout, (void *) task, 
+		irda_start_timer(&task->timer, timeout, (void *) task,
 				 irda_task_timer_expired);
 		finished = FALSE;
 	} else {
-		IRDA_DEBUG(0, __FUNCTION__ 
+		IRDA_DEBUG(0, __FUNCTION__
 			   "(), not finished, and no timeout!\n");
 		finished = FALSE;
 	}
@@ -375,7 +312,7 @@ int irda_task_kick(struct irda_task *task)
  *    This function registers and tries to execute tasks that may take some
  *    time to complete. We do it this hairy way since we may have been
  *    called from interrupt context, so it's not possible to use
- *    schedule_timeout() 
+ *    schedule_timeout()
  * Two important notes :
  *	o Make sure you irda_task_delete(task); in case you delete the
  *	  calling instance.
@@ -383,9 +320,9 @@ int irda_task_kick(struct irda_task *task)
  *	  want to lock within the task handler.
  * Jean II
  */
-struct irda_task *irda_task_execute(void *instance, 
-				    IRDA_TASK_CALLBACK function, 
-				    IRDA_TASK_CALLBACK finished, 
+struct irda_task *irda_task_execute(void *instance,
+				    IRDA_TASK_CALLBACK function,
+				    IRDA_TASK_CALLBACK finished,
 				    struct irda_task *parent, void *param)
 {
 	struct irda_task *task;
@@ -402,7 +339,7 @@ struct irda_task *irda_task_execute(void *instance,
 	task->function = function;
 	task->finished = finished;
 	task->parent   = parent;
-	task->param    = param;	
+	task->param    = param;
 	task->magic    = IRDA_TASK_MAGIC;
 
 	init_timer(&task->timer);
@@ -441,7 +378,7 @@ static void irda_task_timer_expired(void *data)
  *    This function should be used by low level device drivers in a similar way
  *    as ether_setup() is used by normal network device drivers
  */
-int irda_device_setup(struct net_device *dev) 
+int irda_device_setup(struct net_device *dev)
 {
 	ASSERT(dev != NULL, return -1;);
 
@@ -453,7 +390,7 @@ int irda_device_setup(struct net_device *dev)
 
         dev->type            = ARPHRD_IRDA;
         dev->tx_queue_len    = 8; /* Window size + 1 s-frame */
- 
+
 	memset(dev->broadcast, 0xff, 4);
 
 	dev->mtu = 2048;
@@ -499,9 +436,9 @@ dongle_t *irda_device_dongle_init(struct net_device *dev, int type)
 	sprintf(modname, "irda-dongle-%d", type);
 	request_module(modname);
 	}
-#endif /* CONFIG_KMOD */
+#endif
 
-	if (!(reg = hashbin_find(dongles, type, NULL))) {
+	if (!(reg = hashbin_lock_find(dongles, type, NULL))) {
 		ERROR("IrDA: Unable to find requested dongle\n");
 		return NULL;
 	}
@@ -522,9 +459,6 @@ dongle_t *irda_device_dongle_init(struct net_device *dev, int type)
 
 /*
  * Function irda_device_dongle_cleanup (dongle)
- *
- *    
- *
  */
 int irda_device_dongle_cleanup(dongle_t *dongle)
 {
@@ -539,21 +473,18 @@ int irda_device_dongle_cleanup(dongle_t *dongle)
 
 /*
  * Function irda_device_register_dongle (dongle)
- *
- *    
- *
  */
 int irda_device_register_dongle(struct dongle_reg *new)
 {
 	/* Check if this dongle has been registred before */
-	if (hashbin_find(dongles, new->type, NULL)) {
-		MESSAGE(__FUNCTION__ "(), Dongle already registered\n");
+	if (hashbin_lock_find(dongles, new->type, NULL)) {
+		MESSAGE("%s: Dongle already registered\n", __FUNCTION__);
                 return 0;
         }
-	
+
 	/* Insert IrDA dongle into hashbin */
 	hashbin_insert(dongles, (irda_queue_t *) new, new->type, NULL);
-	
+
         return 0;
 }
 
@@ -569,7 +500,7 @@ void irda_device_unregister_dongle(struct dongle_reg *dongle)
 
 	node = hashbin_remove(dongles, dongle->type, NULL);
 	if (!node) {
-		ERROR(__FUNCTION__ "(), dongle not found!\n");
+		ERROR("%s: dongle not found!\n", __FUNCTION__);
 		return;
 	}
 }
@@ -582,22 +513,22 @@ void irda_device_unregister_dongle(struct dongle_reg *dongle)
  *    driver to find out which modes it support.
  */
 int irda_device_set_mode(struct net_device* dev, int mode)
-{	
+{
 	struct if_irda_req req;
 	int ret;
 
 	IRDA_DEBUG(0, __FUNCTION__ "()\n");
 
 	if (!dev->do_ioctl) {
-		ERROR(__FUNCTION__ "(), set_raw_mode not impl. by "
-		      "device driver\n");
+		ERROR("%s: set_raw_mode not impl. by device driver\n",
+				__FUNCTION__);
 		return -1;
 	}
-	
+
 	req.ifr_mode = mode;
 
 	ret = dev->do_ioctl(dev, (struct ifreq *) &req, SIOCSMODE);
-	
+
 	return ret;
 }
 
@@ -610,13 +541,13 @@ int irda_device_set_mode(struct net_device* dev, int mode)
 void setup_dma(int channel, char *buffer, int count, int mode)
 {
 	unsigned long flags;
-	
+
 	flags = claim_dma_lock();
-	
+
 	disable_dma(channel);
 	clear_dma_ff(channel);
 	set_dma_mode(channel, mode);
-	set_dma_addr(channel, virt_to_bus(buffer));
+	set_dma_addr(channel, isa_virt_to_bus(buffer));
 	set_dma_count(channel, count);
 	enable_dma(channel);
 
